@@ -1,20 +1,22 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2011 Edgewall Software
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2022 the Babel team
 # All rights reserved.
 #
-# This software is licensed as described in the file COPYING, which
+# This software is licensed as described in the file LICENSE, which
 # you should have received as part of this distribution. The terms
 # are also available at http://babel.edgewall.org/wiki/License.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://babel.edgewall.org/log/.
+import decimal
 import unittest
 import pytest
 
-from babel import plural
-from babel._compat import Decimal
+from babel import plural, localedata
+
+EPSILON = decimal.Decimal("0.0001")
 
 
 def test_plural_rule():
@@ -34,29 +36,29 @@ def test_plural_rule_operands_i():
 
 def test_plural_rule_operands_v():
     rule = plural.PluralRule({'one': 'v is 2'})
-    assert rule(Decimal('1.20')) == 'one'
-    assert rule(Decimal('1.2')) == 'other'
+    assert rule(decimal.Decimal('1.20')) == 'one'
+    assert rule(decimal.Decimal('1.2')) == 'other'
     assert rule(2) == 'other'
 
 
 def test_plural_rule_operands_w():
     rule = plural.PluralRule({'one': 'w is 2'})
-    assert rule(Decimal('1.23')) == 'one'
-    assert rule(Decimal('1.20')) == 'other'
+    assert rule(decimal.Decimal('1.23')) == 'one'
+    assert rule(decimal.Decimal('1.20')) == 'other'
     assert rule(1.2) == 'other'
 
 
 def test_plural_rule_operands_f():
     rule = plural.PluralRule({'one': 'f is 20'})
-    assert rule(Decimal('1.23')) == 'other'
-    assert rule(Decimal('1.20')) == 'one'
+    assert rule(decimal.Decimal('1.23')) == 'other'
+    assert rule(decimal.Decimal('1.20')) == 'one'
     assert rule(1.2) == 'other'
 
 
 def test_plural_rule_operands_t():
     rule = plural.PluralRule({'one': 't = 5'})
-    assert rule(Decimal('1.53')) == 'other'
-    assert rule(Decimal('1.50')) == 'one'
+    assert rule(decimal.Decimal('1.53')) == 'other'
+    assert rule(decimal.Decimal('1.50')) == 'one'
     assert rule(1.5) == 'one'
 
 
@@ -82,7 +84,7 @@ def test_to_python():
 
 def test_to_gettext():
     assert (plural.to_gettext({'one': 'n is 1', 'two': 'n is 2'})
-            == 'nplurals=3; plural=((n == 1) ? 0 : (n == 2) ? 1 : 2)')
+            == 'nplurals=3; plural=((n == 1) ? 0 : (n == 2) ? 1 : 2);')
 
 
 def test_in_range_list():
@@ -118,7 +120,7 @@ def test_plural_within_rules():
         " ? 'few' : (n == 1) ? 'one' : 'other'; })")
     assert plural.to_gettext(p) == (
         'nplurals=3; plural=(((n == 2) || (n == 4) || (n >= 7 && n <= 9))'
-        ' ? 1 : (n == 1) ? 0 : 2)')
+        ' ? 1 : (n == 1) ? 0 : 2);')
     assert p(0) == 'other'
     assert p(1) == 'one'
     assert p(2) == 'few'
@@ -145,7 +147,7 @@ WELL_FORMED_TOKEN_TESTS = (
     ('n = 1 @integer 1', [('value', '1'), ('symbol', '='), ('word', 'n'), ]),
     ('n is 1', [('value', '1'), ('word', 'is'), ('word', 'n'), ]),
     ('n % 100 = 3..10', [('value', '10'), ('ellipsis', '..'), ('value', '3'),
-                         ('symbol', '='),  ('value', '100'), ('symbol', '%'),
+                         ('symbol', '='), ('value', '100'), ('symbol', '%'),
                          ('word', 'n'), ]),
 )
 
@@ -167,6 +169,7 @@ def test_tokenize_malformed(rule_text):
 
 
 class TestNextTokenTestCase(unittest.TestCase):
+
     def test_empty(self):
         assert not plural.test_next_token([], '')
 
@@ -197,6 +200,7 @@ def make_range_list(*values):
 
 
 class PluralRuleParserTestCase(unittest.TestCase):
+
     def setUp(self):
         self.n = plural.ident_node('n')
 
@@ -238,12 +242,12 @@ class PluralRuleParserTestCase(unittest.TestCase):
 
 EXTRACT_OPERANDS_TESTS = (
     (1, 1, 1, 0, 0, 0, 0),
-    ('1.0', '1.0', 1, 1, 0, 0, 0),
-    ('1.00', '1.00', 1, 2, 0, 0, 0),
-    ('1.3', '1.3', 1, 1, 1, 3, 3),
-    ('1.30', '1.30', 1, 2, 1, 30, 3),
-    ('1.03', '1.03', 1, 2, 2, 3, 3),
-    ('1.230', '1.230', 1, 3, 2, 230, 23),
+    (decimal.Decimal('1.0'), '1.0', 1, 1, 0, 0, 0),
+    (decimal.Decimal('1.00'), '1.00', 1, 2, 0, 0, 0),
+    (decimal.Decimal('1.3'), '1.3', 1, 1, 1, 3, 3),
+    (decimal.Decimal('1.30'), '1.30', 1, 2, 1, 30, 3),
+    (decimal.Decimal('1.03'), '1.03', 1, 2, 2, 3, 3),
+    (decimal.Decimal('1.230'), '1.230', 1, 3, 2, 230, 23),
     (-1, 1, 1, 0, 0, 0, 0),
     (1.3, '1.3', 1, 1, 1, 3, 3),
 )
@@ -251,6 +255,26 @@ EXTRACT_OPERANDS_TESTS = (
 
 @pytest.mark.parametrize('source,n,i,v,w,f,t', EXTRACT_OPERANDS_TESTS)
 def test_extract_operands(source, n, i, v, w, f, t):
-    source = Decimal(source) if isinstance(source, str) else source
-    assert (plural.extract_operands(source) ==
-            Decimal(n), i, v, w, f, t)
+    e_n, e_i, e_v, e_w, e_f, e_t, e_c, e_e = plural.extract_operands(source)
+    assert abs(e_n - decimal.Decimal(n)) <= EPSILON  # float-decimal conversion inaccuracy
+    assert e_i == i
+    assert e_v == v
+    assert e_w == w
+    assert e_f == f
+    assert e_t == t
+    assert not e_c  # Not supported at present
+    assert not e_e  # Not supported at present
+
+
+@pytest.mark.parametrize('locale', ('ru', 'pl'))
+def test_gettext_compilation(locale):
+    # Test that new plural form elements introduced in recent CLDR versions
+    # are compiled "down" to `n` when emitting Gettext rules.
+    ru_rules = localedata.load(locale)['plural_form'].rules
+    chars = 'ivwft'
+    # Test that these rules are valid for this test; i.e. that they contain at least one
+    # of the gettext-unsupported characters.
+    assert any((" " + ch + " ") in rule for ch in chars for rule in ru_rules.values())
+    # Then test that the generated value indeed does not contain these.
+    ru_rules_gettext = plural.to_gettext(ru_rules)
+    assert not any(ch in ru_rules_gettext for ch in chars)
