@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2011 Edgewall Software, 2013-2020 the Babel team
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2022 the Babel team
 # All rights reserved.
 #
 # This software is licensed as described in the file LICENSE, which
@@ -17,15 +17,16 @@ import shutil
 import tempfile
 import unittest
 import pytest
+import sys
 from datetime import date, datetime, timedelta
+from io import BytesIO
 
 from babel import support
 from babel.messages import Catalog
 from babel.messages.mofile import write_mo
-from babel._compat import BytesIO, PY2
 
-get_arg_spec = (inspect.getargspec if PY2 else inspect.getfullargspec)
 
+SKIP_LGETTEXT = sys.version_info >= (3, 8)
 
 @pytest.mark.usefixtures("os_environ")
 class TranslationsTestCase(unittest.TestCase):
@@ -76,6 +77,7 @@ class TranslationsTestCase(unittest.TestCase):
         self.assertEqualTypeToo(u'VohCTX', self.translations.upgettext('foo',
                                                                        'foo'))
 
+    @pytest.mark.skipif(SKIP_LGETTEXT, reason="lgettext is deprecated")
     def test_lpgettext(self):
         self.assertEqualTypeToo(b'Voh', self.translations.lgettext('foo'))
         self.assertEqualTypeToo(b'VohCTX', self.translations.lpgettext('foo',
@@ -105,6 +107,7 @@ class TranslationsTestCase(unittest.TestCase):
                                 self.translations.unpgettext('foo', 'foo1',
                                                              'foos1', 2))
 
+    @pytest.mark.skipif(SKIP_LGETTEXT, reason="lgettext is deprecated")
     def test_lnpgettext(self):
         self.assertEqualTypeToo(b'Voh1',
                                 self.translations.lngettext('foo1', 'foos1', 1))
@@ -129,6 +132,7 @@ class TranslationsTestCase(unittest.TestCase):
         self.assertEqualTypeToo(
             u'VohCTXD', self.translations.dupgettext('messages1', 'foo', 'foo'))
 
+    @pytest.mark.skipif(SKIP_LGETTEXT, reason="lgettext is deprecated")
     def test_ldpgettext(self):
         self.assertEqualTypeToo(
             b'VohD', self.translations.ldgettext('messages1', 'foo'))
@@ -159,6 +163,7 @@ class TranslationsTestCase(unittest.TestCase):
             u'VohsCTXD1', self.translations.dunpgettext('messages1', 'foo', 'foo1',
                                                         'foos1', 2))
 
+    @pytest.mark.skipif(SKIP_LGETTEXT, reason="lgettext is deprecated")
     def test_ldnpgettext(self):
         self.assertEqualTypeToo(
             b'VohD1', self.translations.ldngettext('messages1', 'foo1', 'foos1', 1))
@@ -197,7 +202,11 @@ class NullTranslationsTestCase(unittest.TestCase):
         self.null_translations = support.NullTranslations(fp=fp)
 
     def method_names(self):
-        return [name for name in dir(self.translations) if 'gettext' in name]
+        names = [name for name in dir(self.translations) if 'gettext' in name]
+        if SKIP_LGETTEXT:
+            # Remove deprecated l*gettext functions
+            names = [name for name in names if not name.startswith('l')]
+        return names
 
     def test_same_methods(self):
         for name in self.method_names():
@@ -209,8 +218,8 @@ class NullTranslationsTestCase(unittest.TestCase):
             translations_method = getattr(self.translations, name)
             null_method = getattr(self.null_translations, name)
             self.assertEqual(
-                get_arg_spec(translations_method),
-                get_arg_spec(null_method),
+                inspect.getfullargspec(translations_method),
+                inspect.getfullargspec(null_method),
             )
 
     def test_same_return_values(self):
@@ -222,7 +231,7 @@ class NullTranslationsTestCase(unittest.TestCase):
         for name in self.method_names():
             method = getattr(self.translations, name)
             null_method = getattr(self.null_translations, name)
-            signature = get_arg_spec(method)
+            signature = inspect.getfullargspec(method)
             parameter_names = [name for name in signature.args if name != 'self']
             values = [data[name] for name in parameter_names]
             self.assertEqual(method(*values), null_method(*values))
@@ -279,6 +288,17 @@ class LazyProxyTestCase(unittest.TestCase):
         self.assertEqual(2, proxy.value)
         self.assertEqual(1, proxy_deepcopy.value)
 
+    def test_handle_attribute_error(self):
+
+        def raise_attribute_error():
+            raise AttributeError('message')
+
+        proxy = support.LazyProxy(raise_attribute_error)
+        with pytest.raises(AttributeError) as exception:
+            proxy.value
+
+        self.assertEqual('message', str(exception.value))
+
 
 def test_format_date():
     fmt = support.Format('en_US')
@@ -344,15 +364,7 @@ def test_catalog_merge_files():
     t1 = support.Translations()
     assert t1.files == []
     t1._catalog["foo"] = "bar"
-    if PY2:
-        # Explicitly use the pure-Python `StringIO` class, as we need to
-        # augment it with the `name` attribute, which we can't do for
-        # `babel._compat.BytesIO`, which is `cStringIO.StringIO` under
-        # `PY2`...
-        from StringIO import StringIO
-        fp = StringIO()
-    else:
-        fp = BytesIO()
+    fp = BytesIO()
     write_mo(fp, Catalog())
     fp.seek(0)
     fp.name = "pro.mo"
