@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2011 Edgewall Software, 2013-2021 the Babel team
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2022 the Babel team
 # All rights reserved.
 #
 # This software is licensed as described in the file LICENSE, which
@@ -13,11 +13,11 @@
 
 import __future__
 import unittest
+from io import BytesIO
 
 import pytest
 
 from babel import util
-from babel._compat import BytesIO
 from babel.util import parse_future_flags
 
 
